@@ -105,6 +105,8 @@ sub _load_sandbox {
     my $_package = $_file;
     $_package =~ s/([^A-Za-z0-9_])/sprintf("_%2x", unpack("C", $1))/eg;
 
+    _file_zero_check($_file) if $ENV{PLACK_ENV} eq 'development';
+
     local $0 = $_file; # so FindBin etc. works
 
     return eval sprintf <<'END_EVAL', $_package;
@@ -117,6 +119,27 @@ package Plack::Sandbox::%s;
 END_EVAL
 }
 
+sub _file_zero_check {
+    my $file = shift;
+    open my $fh, "<", $file or return;
+
+    my $code = join '', <$fh>;
+    if ($code =~ /(__FILE__\s+eq\s+\$0|\$0\s+eq\+__FILE__)/) {
+        warn <<WARNING
+Your PSGI file ($file) seems to use the following idiom, which is known to be broken since Plack 0.9971:
+
+  if ($1) {
+      called_from_cmdline();
+  }
+
+because now \$0 is _always_ localized to the PSGI file path you're evaluating. You should switch to other alternatives such as `unless (caller) {}`. See http://bit.ly/psgi-file-0 for details.
+
+This friendly warning and the code to generate this runs only when the Plack environment (-E) is set to 'development', and will go away in the next major release of Plack.
+
+WARNING
+    }
+}
+
 sub load_psgi {
     my $stuff = shift;
 
